@@ -92,7 +92,7 @@ gb_decl_var (GBParseData *parse_data,
 	new_var->is_array = is_array;
 	new_var->indices  = indices;
 
-	g_hash_table_insert (*variables, &new_var->name, new_var);
+	g_hash_table_insert (*variables, (char *)new_var->name, new_var);
 
 	return FALSE;
 }
@@ -116,7 +116,7 @@ gb_routine_start (GBParseData        *module,
 		  GSList             *args,
 		  const char         *as_type,
 		  gboolean const      is_function,
-		  GBClassPropertyType prop_type)
+		  GBClassMethodType prop_type)
 {
 	GBRoutine *routine;
 
@@ -398,7 +398,7 @@ gb_parse_data_add_var (GBParseData *parse_data,
 		return TRUE;
 	}
 
-	g_hash_table_insert (*variables, &var->name, var);
+	g_hash_table_insert (*variables, (char *)var->name, var);
 
 	return FALSE;
 }
@@ -416,7 +416,7 @@ gb_parse_data_add_const (GBParseData  *parse_data,
 	if (!parse_data->constants)
 		parse_data->constants = gb_priv_new_hash ();
 
-	g_hash_table_insert (parse_data->constants, &cons->name, cons);
+	g_hash_table_insert (parse_data->constants, cons->name, cons);
 }
 
 void
