@@ -14,6 +14,7 @@
 #include <gb/gb.h>
 #include <gb/gb-type.h>
 #include <gb/gb-expr.h>
+#include <gb/gb-class.h>
 
 typedef enum { 
 	GB_COMPARE_BINARY,
@@ -28,12 +29,6 @@ struct _GBOptions {
 	int compare;
 };
 
-typedef enum { 
-	GB_CLASS_PROPERTY_GET,
-	GB_CLASS_PROPERTY_LET,
-	GB_CLASS_NONE
-} GBClassPropertyType;
-
 /* Function/Subroutine */
 struct _GBRoutine {
 	const gchar *name;
@@ -42,7 +37,7 @@ struct _GBRoutine {
 	gboolean     is_function;
 
 	/* For class properties */
-	GBClassPropertyType prop_type;
+	GBClassMethodType prop_type;
 
 	/* Argument descriptions */
 	GSList      *args;
@@ -110,7 +105,7 @@ gboolean         gb_routine_start   (GBParseData        *module,
 				     GSList             *args,
 				     const char         *as_type,
 				     gboolean            is_function,
-				     GBClassPropertyType prop_type);
+				     GBClassMethodType prop_type);
 
 GBRoutine const *gb_routine_finish  (GBParseData *module,
 				     gboolean  is_private,
