@@ -11,6 +11,7 @@
  * Copyright 2000, Helix Code, Inc.
  */
 
+#include <gdk/gdkkeysyms.h>
 #include "gbrun-form-item.h"
 
 #define ITEM_NAME "gb-vb.commandbutton"
@@ -40,36 +41,31 @@ typedef struct {
  *   Callback for 'click'.
  *
  **/
+
+static void
+cmdbutton_mouse_out (GtkWidget *button, GBRunEvalContext *ec)
+{
+	gbrun_form_item_invoke (ec, button, "_MouseOut");
+}
+
+static void
+cmdbutton_mouse_in (GtkWidget *button, GBRunEvalContext *ec)
+{
+	gbrun_form_item_invoke (ec, button, "_MouseIn");
+}
+
+static void
+cmdbutton_key_press (GtkWidget *button, GBRunEvalContext *ec)
+{
+	gbrun_form_item_invoke (ec, button, "_KeyPress");
+}
+
 static void
 cmdbutton_click (GtkWidget *button, GBRunEvalContext *ec)
 {
 	gbrun_form_item_invoke (ec, button, "_Click");
 }
 
-static void
-cmdbutton_mousedown (GtkWidget *button, GBRunEvalContext *ec)
-{
-	gbrun_form_item_invoke (ec, button, "_MouseDown");
-}
-
-static void
-cmdbutton_mouseup (GtkWidget *button, GBRunEvalContext *ec)
-{
-	gbrun_form_item_invoke (ec, button, "_MouseUp");
-}
-
-static void
-cmdbutton_mouseover (GtkWidget *button, GBRunEvalContext *ec)
-{
-	gbrun_form_item_invoke (ec, button, "_MouseOver");
-}
-
-static void
-cmdbutton_mouseout (GtkWidget *button, GBRunEvalContext *ec)
-{
-	gbrun_form_item_invoke (ec, button, "_MouseOut");
-}
-
 /**
  * cmdbutton_setarg:
  *   @ec
@@ -93,11 +89,26 @@ cmdbutton_setarg (GBRunEvalContext *ec,
 	case CAPTION:
 	{
 		char *txt;
+		guint keyval;
 
 		button->real_caption = g_strdup (val->v.s->str);
 
 		txt = gbrun_form_un_shortcutify (button->real_caption, NULL);
 		gtk_label_set_text (button->label, txt);
+
+		gtk_accel_label_set_accel_widget (
+			GTK_ACCEL_LABEL (button->label), bwidget);
+
+		keyval = gtk_label_parse_uline (GTK_LABEL (button->label), txt);
+
+		if (keyval != GDK_VoidSymbol &&
+		    button->item.form)
+			gtk_widget_add_accelerator (
+				bwidget, "clicked",
+				button->item.form->accel_group,
+				keyval, GDK_MOD1_MASK,
+				(GtkAccelFlags) 0);
+		
 		g_free (txt);
 
 		return TRUE;
@@ -183,23 +194,25 @@ gbrun_cmdbutton_construct (GBRunEvalContext *ec, GBRunFormItem *item)
 
 	w = gtk_button_new ();
 	gbrun_form_item_set_widget (GBRUN_FORM_ITEM (dest), w);
-	dest->label = GTK_LABEL (gtk_label_new ("WIBBLE"));
+	dest->label = GTK_LABEL (gtk_accel_label_new ("WIBBLE"));
 	gtk_widget_show (GTK_WIDGET (dest->label));
 
 	gtk_container_add (GTK_CONTAINER (w),
 			   GTK_WIDGET    (dest->label));
 
 	/* FIXME: more signals etc. & leak fix */
-	gtk_signal_connect (GTK_OBJECT (w), "clicked",
+
+	gtk_signal_connect (GTK_OBJECT (w), "enter", 
+				(GtkSignalFunc) cmdbutton_mouse_in, ec);
+
+	gtk_signal_connect (GTK_OBJECT (w), "leave", 
+				(GtkSignalFunc) cmdbutton_mouse_out, ec);
+
+	gtk_signal_connect (GTK_OBJECT (w), "pressed", 
+				(GtkSignalFunc) cmdbutton_key_press, ec);
+
+	gtk_signal_connect (GTK_OBJECT (item->widget), "clicked", 
 			    (GtkSignalFunc) cmdbutton_click, ec);
-	gtk_signal_connect (GTK_OBJECT (w), "pressed",
-				(GtkSignalFunc) cmdbutton_mousedown, ec);
-	gtk_signal_connect (GTK_OBJECT (w), "released",
-				(GtkSignalFunc) cmdbutton_mouseup, ec);
-	gtk_signal_connect (GTK_OBJECT (w), "enter",
-				(GtkSignalFunc) cmdbutton_mouseover, ec);
-	gtk_signal_connect (GTK_OBJECT (w), "leave",
-				(GtkSignalFunc) cmdbutton_mouseout, ec);
 }
 
 static void
@@ -225,7 +238,7 @@ gbrun_cmdbutton_class_init (GBRunObjectClass *klass)
 
 	form_class->construct = gbrun_cmdbutton_construct;
 
-	gtk_class->destroy = gbrun_cmdbutton_destroy;
+	gtk_class->destroy    = gbrun_cmdbutton_destroy;
 }
 
 GtkType
