@@ -24,6 +24,7 @@ gbrun_objects_register (GBEvalContext *ec)
 	gbrun_textbox_register ();
 	gbrun_label_register ();
 	gbrun_checkbox_register();
+	gbrun_menu_register ();
 	gbrun_timer_register ();
 	gbrun_picturebox_register ();
 	gbrun_listbox_register ();
@@ -40,6 +41,7 @@ gbrun_objects_shutdown (void)
 	gbrun_checkbox_shutdown ();
 	gbrun_picturebox_shutdown ();
 	gbrun_timer_shutdown ();
+	gbrun_menu_shutdown ();
 	gbrun_textbox_shutdown ();
 	gbrun_label_shutdown ();
 	gbrun_cmdbutton_shutdown ();
