@@ -20,6 +20,7 @@ GtkType gbrun_timer_get_type     (void);
 GtkType gbrun_picturebox_get_type(void);
 GtkType gbrun_frame_get_type     (void);
 GtkType gbrun_checkbox_get_type  (void);
+GtkType gbrun_menu_get_type      (void);
 GtkType gbrun_screen_get_type    (void);
 
 void gbrun_cmdbutton_register (void);
@@ -30,6 +31,7 @@ void gbrun_timer_register     (void);
 void gbrun_picturebox_register(void);
 void gbrun_frame_register     (void);
 void gbrun_checkbox_register  (void);
+void gbrun_menu_register      (void);
 void gbrun_screen_register    (void);
 
 void gbrun_cmdbutton_shutdown (void);
@@ -40,9 +42,9 @@ void gbrun_timer_shutdown     (void);
 void gbrun_picturebox_shutdown(void);
 void gbrun_frame_shutdown     (void);
 void gbrun_checkbox_shutdown  (void);
+void gbrun_menu_shutdown      (void);
 void gbrun_screen_shutdown    (void);
 
-
 /* External */
 void gbrun_objects_register   (GBEvalContext *ec);
 void gbrun_objects_shutdown   (void);
